import greenfoot.*;
import java.awt.*;
import java.util.List;

public class LightingOverlay extends Actor
{
    int baseAlpha = 255;
    public void addedToWorld(World world){
        GreenfootImage base = new GreenfootImage(getWorld().getWidth(),getWorld().getHeight());
        base.setColor(new Color(0,0,0,baseAlpha));
        base.fill();
        setImage(base);
        
        setLocation(getWorld().getWidth()/2,getWorld().getHeight()/2);
    }
    boolean glitchy = false;
    boolean ghold = false;
    public void act()
    {
        //change base alpha
        if (!ghold&&Greenfoot.isKeyDown("g")){
            glitchy=!glitchy;
            ghold=true;
        }else if(ghold&&!Greenfoot.isKeyDown("g")){
            ghold=false;
        }
        if (Greenfoot.isKeyDown("="))baseAlpha++;
        if (Greenfoot.isKeyDown("-"))baseAlpha--;
        if (baseAlpha<0)baseAlpha=0;
        if (baseAlpha>255)baseAlpha=255;
        //reset image
        
        if (glitchy){
            GreenfootImage image = getImage();
            image.setColor(new Color(0,0,0,baseAlpha));
            image.fill();
        }else{
            GreenfootImage image = new GreenfootImage(getWorld().getWidth(),getWorld().getHeight());
            image.setColor(new Color(0,0,0,baseAlpha));
            image.fill();
            setImage(image);
        }
        
        //Do lighting
        List matter = getWorld().getObjects(Matter.class);
        if (!matter.isEmpty()){
            for (int i=0; i<matter.size(); i++){
                Actor object = (Matter) matter.get(i);
                lightPlayerView(object.getX(), object.getY(), ((Matter)object).getLuminosity(), ((Matter)object).getRedLight(), ((Matter)object).getGreenLight(), ((Matter)object).getBlueLight());
            }
        }
        //Make objects block light
        List objects = getWorld().getObjects(LevelObjects.class);
        if (!objects.isEmpty()){
            for (int i=0; i<objects.size(); i++){
                Actor object = (LevelObjects) objects.get(i);
                blockLight(object.getX(),object.getY(),((LevelObjects)object).getAntiLuminosity(),object.getImage());
            }
        }
    }
    
    public void lightPlayerView(int x, int y, int luminosity, int redLight, int greenLight, int blueLight)
    {
        GreenfootImage image = new GreenfootImage(getImage());
        //These for loops will go through every single point on the screen
        for (int i=0; i<image.getWidth(); i++){
            for (int j=0; j<image.getHeight(); j++){
                int intensity = luminosity - (int) Math.sqrt((i-x)*(i-x)+(j-y)*(j-y)); //how close point of lighting is to the origin of light
                if (intensity>0){ //if within lighting range
                    Color c = image.getColorAt(i,j);//get color at point
                    int r = c.getRed(), g = c.getGreen(), b = c.getBlue(), a = c.getAlpha();//get color values
                    int alpha = a - intensity;//brighten point (yes light amount is the same as how far away it is from the light origin)
                    //proportionally fade colors (so it darkens into black)
                    r += redLight*intensity/luminosity;
                    g += greenLight*intensity/luminosity;
                    b += blueLight*intensity/luminosity;
                    //Handle out of bounds values
                    if (alpha>255)alpha=255;
                    if (alpha<0)alpha=0;
                    if (r>255)r=255;
                    if (r<0)r=0;
                    if (g>255)g=255;
                    if (g<0)g=0;
                    if (b>255)b=255;
                    if (b<0)b=0;
                    //change color
                    image.setColorAt(i,j,new Color(r,g,b,alpha));
                }
            }
        }
        setImage(image);//set image to updated version
    }
    public void blockLight(int x, int y, int antiLuminosity, GreenfootImage blocker){
        GreenfootImage image = new GreenfootImage(getImage());
        int width = blocker.getWidth(), height = blocker.getHeight();
        //find blocker image coordinates (which are the at the top-left point of the blocker's image)
        x-=width/2;
        y-=height/2;
        //Find every single point where blocker is on top of image in image
        for (int i=0; i<width; i++){
            for (int j=0; j<height; j++){
                if (x+i>0&&x+i<getImage().getWidth()&&y+j>0&&y+j<getWorld().getHeight()&&blocker.getColorAt(i,j).getAlpha()>0){//Make sure point is on screen, and that it falls on a point on the object image that isn't 100% transparent
                    Color c = image.getColorAt(x+i,y+j);
                    if (c.getAlpha()<255){//make sure color isn't 100% visible, or else / 0 error (rgb are 0 if alpha is 0 anyway)
                        int r = c.getRed(), g = c.getGreen(), b = c.getBlue(), a = c.getAlpha();
                        int alpha = a+antiLuminosity;
                        //Proportionally fade color values
                        r = r*(255-alpha)/(255-a);
                        g = g*(255-alpha)/(255-a);
                        b = b*(255-alpha)/(255-a);
                        if (alpha>255)alpha=255;
                        if (alpha<0)alpha=0;
                        if (r>255)r=255;
                        if (r<0)r=0;
                        if (g>255)g=255;
                        if (g<0)g=0;
                        if (b>255)b=255;
                        if (b<0)b=0;
                        image.setColorAt(x+i,y+j,new Color(r,g,b,alpha));
                    }
                }
            }
        }
        setImage(image);
    }
    
    public void setBaseAlpha(int alpha){
        baseAlpha = alpha;
    }
    public int getBaseAlpha(){
        return baseAlpha;
    }
}